@@ -1,6 +1,6 @@
 package TPath::Context;
 {
-  $TPath::Context::VERSION = '1.002';
+  $TPath::Context::VERSION = '1.003';
 }
 
 # ABSTRACT: the context in which a node is evaluated during a search
@@ -27,14 +27,14 @@ sub new {
 sub bud {
 
     # my ( $self, $n ) = @_;
-    return bless [ $_[1], $_[0]->[1], [ $_[0]->[0], @{ $_[0]->[2] } ] ];
+    return bless [ $_[1], $_[0][1], [ $_[0][0], @{ $_[0][2] } ] ];
 }
 
 #Makes a context that doesn't preserve the path.
 sub wrap {
 
     # my ( $self, $n ) = @_;
-    return bless [ $_[1], $_[0]->[1], [] ];
+    return bless [ $_[1], $_[0][1], [] ];
 }
 
 
@@ -63,10 +63,17 @@ sub i { $_[0][1] }
 sub path { $_[0][2] }
 
 
+sub expression {
+    return $_[0][3] if @_ < 2;
+    my ( $self, $e ) = @_;
+    return $self->[3] = $e;
+}
+
+
 sub to_string {
     my $s;
     eval { $s = "$_[0][0]" };
-    if ($@) {    # workaround for odd overload bug
+    if ($@) {                  # workaround for odd overload bug
         $s = 'memaddr' . refaddr $_[0][0];
     }
     return $s;
@@ -84,7 +91,7 @@ TPath::Context - the context in which a node is evaluated during a search
 
 =head1 VERSION
 
-version 1.002
+version 1.003
 
 =head1 DESCRIPTION
 
@@ -93,7 +100,9 @@ to selectors, predicates, or attributes during the evaluation of a node. This cl
 method signatures -- instead of passing a list of parameters one passes a single context.
 
 A C<TPath::Context> is a blessed array rather than a hash, and it is a non-Moose class, for a 
-little added efficiency.
+little added efficiency. Note, that for still greater efficiency it is sometimes treated as an
+array rather than an object, so it must be regarded as a final class not to be tampered with
+or extended.
 
 =head1 METHODS
 
@@ -118,6 +127,14 @@ The L<TPath::Index>.
 The previous nodes selected in the course of selecting the context node. These ancestor
 nodes are in reverse order, so the node's immediate predecessor is at index 0.
 
+=head2 expression
+
+The expression the context is being used by. This attribute is not guaranteed to be set and
+the method, unlike the other accessors, is a setter as well as a getter.
+
+The contextual attribute is available for use by L<TPath::Attributes>, which set the attribute
+when they are applied to a context.
+
 =head2 to_string
 
 The stringification of a context is the stringification of its node.
